class APIError(Exception):
    pass


class BadRequest(APIError):
    pass


class QuotaError(APIError):
    pass


class NotFoundError(APIError):
    pass


class MethodNotAllowed(APIError):
    pass


class ForbiddenError(APIError):
    pass


class TooManyRequestsError(APIError):
    pass


class InternalServerError(APIError):
    pass


class ServiceUnavailable(APIError):
    pass


class GatewayTimeout(APIError):
    pass



from typing import Type

from pydantic import BaseModel

from ..errors import (
    BadRequest,
    ForbiddenError,
    GatewayTimeout,
    InternalServerError,
    MethodNotAllowed,
    NotFoundError,
    QuotaError,
    ServiceUnavailable,
    TooManyRequestsError,
)

errors = {
    400: BadRequest,
    402: QuotaError,
    403: ForbiddenError,
    404: NotFoundError,
    405: MethodNotAllowed,
    429: TooManyRequestsError,
    500: InternalServerError,
    503: ServiceUnavailable,
    504: GatewayTimeout,
}


async def check_response(status, response: dict, datatype: Type[BaseModel]):
    if status == 200:
        return datatype(**response)
    exception = errors[status]
    raise exception(response["error"])



from typing import List, Optional, Union

from pydantic import BaseModel
from keyword import kwlist


def to_calmel_case(string: str) -> str:
    if string in kwlist:
        return "_" + string
    words = string.split("_")
    return words[0] + "".join(word.capitalize() for word in words[1:])


BaseModel.Config.alias_generator = to_calmel_case
BaseModel.Config.allow_population_by_field_name = True


class Error(BaseModel):
    """Error object"""

    error: str
    """Error message"""


class User(Error):
    """User object"""

    id: str
    """IP address (guest) or email address (user)"""
    priority: int
    """Your priority in search queue"""
    concurrency: int
    """Number of parallel search requests you can make"""
    quota: int
    """Search quota you have for this month"""
    quota_used: int
    """Search quota you have used this month"""


class AnilistTitle(BaseModel):
    native: Optional[str]
    romaji: Optional[str]
    english: Optional[str]


class Anilist(BaseModel):
    id: int
    id_mal: int
    is_adult: bool
    synonyms: List[str]
    title: AnilistTitle


class Result(BaseModel):
    """Result object"""

    anilist: Union[int, Anilist]
    """The matching Anilist ID or Anilist info"""
    filename: str
    """The filename of file where the match is found"""
    episode: Optional[Union[int, float, str, List[Union[int, float, str]]]]
    """The extracted episode number from filename"""
    _from: float
    """Starting time of the matching scene (seconds)"""
    to: float
    """Ending time of the matching scene (seconds)"""
    similarity: float
    """Similarity compared to the search image"""
    video: str
    """URL to the preview video of the matching scene"""
    image: str
    """URL to the preview image of the matching scene"""


class SearchResult(Error):
    """SearchResult object"""

    frame_count: int
    """Frames compared for image search"""
    result: List[Result]
    """Search results"""


Error.update_forward_refs()
SearchResult.update_forward_refs()
Result.update_forward_refs()
Anilist.update_forward_refs()



from .errors import (
    APIError,
    BadRequest,
    ForbiddenError,
    GatewayTimeout,
    InternalServerError,
    MethodNotAllowed,
    NotFoundError,
    QuotaError,
    ServiceUnavailable,
    TooManyRequestsError,
)

__all__ = [
    "APIError",
    "BadRequest",
    "ForbiddenError",
    "GatewayTimeout",
    "InternalServerError",
    "MethodNotAllowed",
    "NotFoundError",
    "QuotaError",
    "ServiceUnavailable",
    "TooManyRequestsError",
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aiomoe', 'aiomoe.api', 'aiomoe.errors', 'aiomoe.models', 'aiomoe.utils']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.4,<4.0.0', 'loguru>=0.5.3,<0.6.0', 'pydantic>=1.8.2,<2.0.0']

setup_kwargs = {
    'name': 'aiomoe',
    'version': '1.0.3',
    'description': 'Fully asynchronous trace.moe API wrapper',
    'long_description': '# AioMoe\n\nFully asynchronous trace.moe API wrapper\n\n## Installation\n\nYou can install the stable version from PyPI:\n\n    $ pip install aiomoe\n\nOr get it from github:\n\n    $ pip install https://github.com/FeeeeK/aiomoe/archive/refs/heads/master.zip\n\n## Usage\n\n### Get info about your account\n\n```python\nimport asyncio\nfrom aiomoe import AioMoe\n\ntm = AioMoe() # or AioMoe(token="xxxxxxxx")\n\nasync def main():\n    me = await tm.me()\n    print(me)\n    print(f"Used quota: {me.quota_used}/{me.quota}")\n\nasyncio.run(main())\n```\nThe output will be like this:\n```\nUser(error=None, id=\'your ip\', priority=0, concurrency=1, quota=1000, quota_used=0)\nUsed quota: 0/1000\n```\n\n### Search anime\n```python\nimport asyncio\nfrom aiomoe import AioMoe\n\ntm = AioMoe()\n\nasync def main():\n    image = "https://i.imgur.com/Xrb06w5.png"\n    search_results = await tm.search(file_source=image, anilist_info=True)\n    print(search_results.result[0].anilist.title.romaji)\n    # \'Steins;Gate 0\'\n\nasyncio.run(main())\n```\nYou can pass a link to an image, bytes or file-like object (`io.BytesIO`)\n```python\n    with open("image.png", "rb") as file:\n        search_results = await tm.search(file)\n```\nAnd use additional parameters such as:\n - anilist_info - Return an `Anilist` object instead of anilist id\n - cut_borders - Cut out black borders from screenshots\n - anilist_id - Filter results by anilist id\n\n## See Also\n  - [Response objects](https://github.com/FeeeeK/aiomoe/blob/master/aiomoe/models/models.py)\n  - [trace.moe API docs](https://soruly.github.io/trace.moe-api/#/docs)\n  - [trace.moe API swagger docs](https://app.swaggerhub.com/apis/soruly/api.trace.moe)\n\n## Contributing\n\n1.  Fork it\n2.  Create your feature branch (`git checkout -b my-new-feature`)\n3.  Commit your changes (`git commit -am \'Add some feature\'`)\n4.  Push to the branch (`git push origin my-new-feature`)\n5.  Create new Pull Request\n\n## License\n\nReleased under the MIT license.\n\nCopyright by [FeeeeK](https://github.com/feeeek).\n',
    'author': 'FeeeeK',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/FeeeeK/aiomoe',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



from io import BytesIO
from typing import BinaryIO, Dict, Optional, Union

from aiohttp import ClientSession
from loguru import logger

from ..models import SearchResult, User
from ..utils import check_response


class API:
    """Main class for API calls"""

    API_URL = "https://api.trace.moe"

    def __init__(self, token: str = None, session_kwargs: dict = {}):
        self.token = token
        self.session_kwargs = session_kwargs

    async def me(self) -> User:
        url = self.API_URL + "/me"
        query = {}
        if self.token:
            query["key"] = self.token
        status, response = await self.request(url, query)
        logger.debug(f"Got response from {self.API_URL}/me: {response}")
        return await check_response(status, response, User)

    async def search(
        self,
        file_source: Union[BinaryIO, bytes, str],
        anilist_info: Optional[bool] = False,
        cut_borders: Optional[bool] = False,
        anilist_id: Optional[int] = None,
    ) -> SearchResult:
        """/search

        file_source - Image url or a file-like object
        anilist_info - Return an `Anilist` object instead of anilist id
        cut_borders - Cut out black borders from screenshots
        anilist_id - Filter results by anilist id
        """
        url = self.API_URL + "/search"
        query: Dict[str, Union[str, int]] = {}
        file: Optional[BinaryIO] = None
        if self.token:
            query["key"] = self.token
        if anilist_info:
            query["anilistInfo"] = 1
        if cut_borders:
            query["cutBorders"] = 1
        if anilist_id:
            query["anilistID"] = anilist_id
        if isinstance(file_source, str):
            query["url"] = file_source
        elif isinstance(file_source, bytes):
            file = BytesIO(file_source)
        else:
            file = file_source
        status, response = await self.request(url, query, file)
        logger.debug(f"Got response from {self.API_URL}/search: {response}")
        return await check_response(status, response, SearchResult)

    async def request(self, url: str, query: dict, data: Optional[BinaryIO] = None):
        files = None
        if data:
            files = {"image": data}
        async with ClientSession(**self.session_kwargs) as session, session.request(
            "POST" if data else "GET", url, params=query, data=files
        ) as response:
            logger.debug(f"Request sent to '{url}', status: {response.status}")
            return response.status, await response.json()
